@@ -1,7 +1,7 @@
 /* A utility program for copying files. Similar to 'dd' but using
  * the 'Extended Copy' command.
  *
- *  Copyright (c) 2011-2012 Hannes Reinecke, SUSE Labs
+ *  Copyright (c) 2011-2013 Hannes Reinecke, SUSE Labs
  *
  *  Largely taken from 'sg_dd', which has the
  *
@@ -61,7 +61,7 @@
 #include "sg_cmds_extra.h"
 #include "sg_io_linux.h"
 
-static char * version_str = "0.31 20120919";
+static const char * version_str = "0.34 20130507";
 
 #define ME "sg_xcopy: "
 
@@ -80,7 +80,7 @@ static char * version_str = "0.31 20120919";
 #define CACHING_MP 8
 #define CONTROL_MP 0xa
 
-#define SENSE_BUFF_LEN 32       /* Arbitrary, could be larger */
+#define SENSE_BUFF_LEN 64       /* Arbitrary, could be larger */
 #define READ_CAP_REPLY_LEN 8
 #define RCAP16_REPLY_LEN 32
 
@@ -112,6 +112,9 @@ static char * version_str = "0.31 20120919";
 #define TD_RDMA 128
 #define TD_FW 256
 #define TD_SAS 512
+#define TD_IPV6 1024
+#define TD_IP_COPY_SERVICE 2048
+#define TD_ROD 4096
 
 #define DEV_NULL_MINOR_NUM 3
 
@@ -460,9 +463,10 @@ usage()
            "[seek=SEEK] [skip=SKIP]\n"
            "                 [--help] [--version]\n\n"
            "                 [bpt=BPT] [cat=0|1] [dc=0|1] "
-           "[id_usage=hold|discard]\n"
+           "[id_usage=hold|discard|disable]\n"
            "                 [list_id=ID] [prio=PRIO] [time=0|1] "
            "[verbose=VERB]\n"
+           "                 [--on_dst|--on_src] [--verbose]\n"
            "  where:\n"
            "    bpt         is blocks_per_transfer (default is 128 or 32 "
            "when BS>=2048)\n"
@@ -473,8 +477,9 @@ usage()
            "    dc          segment descriptor DC bit (default: 0)\n"
            "    ibs         input block size (if given must be same as "
            "'bs=')\n"
-           "    id_usage    sets list id usage field to hold (0) or "
-           "discard (2)\n"
+           "    id_usage    sets list id usage field to hold (0), "
+           "discard (2) or\n"
+           "                disable (3)\n"
            "    if          file or device to read from (def: stdin)\n"
            "    iflag       comma separated list from: [cat,dc,excl,"
            "flock,null]\n"
@@ -496,9 +501,14 @@ usage()
            "    verbose     0->quiet(def), 1->some noise, 2->more noise, "
            "etc\n"
            "    --help      print out this usage message then exit\n"
+           "    --on_dst    send XCOPY command to the output file/device\n"
+           "    --on_src    send XCOPY command to the input file/device.\n"
+           "                Default if this and --on_dst options not "
+           "given\n"
+           "    --verbose   same action as verbose=1\n"
            "    --version   print version information then exit\n\n"
-           "copy from IFILE to OFILE, similar to dd command; "
-           "but using the SCSI\nEXTENDED COPY command\n");
+           "Copy from IFILE to OFILE, similar to dd command; "
+           "but using the SCSI\nEXTENDED COPY (XCOPY) command.\n");
 }
 
 static int
@@ -625,16 +635,18 @@ scsi_operating_parameter(struct xcopy_fp_t *xfp, int is_target)
     int res;
     unsigned char rcBuff[256];
     unsigned int rcBuffLen = 256, len, n, td_list = 0;
-    unsigned long num, max_target_num, max_segment_num;
-    unsigned long max_segment_len, max_desc_len, max_inline_data, held_data_limit;
+    unsigned long num, max_target_num, max_segment_num, max_segment_len;
+    unsigned long max_desc_len, max_inline_data, held_data_limit;
     int verb, valid = 0;
 
     verb = (verbose ? verbose - 1: 0);
-    res = sg_ll_receive_copy_results(xfp->sg_fd, 0x03, 0, rcBuff, rcBuffLen, 0, verb);
+    res = sg_ll_receive_copy_results(xfp->sg_fd, 0x03, 0, rcBuff, rcBuffLen,
+                                     0, verb);
     if (0 != res)
         return -res;
 
-    len = (rcBuff[0] << 24) | (rcBuff[1] << 16) | (rcBuff[2] << 8) | rcBuff[3];
+    len = (rcBuff[0] << 24) | (rcBuff[1] << 16) | (rcBuff[2] << 8) |
+          rcBuff[3];
     if (len > rcBuffLen) {
         fprintf(stderr, "  <<report too long for internal buffer,"
                 " output truncated\n");
@@ -645,11 +657,13 @@ scsi_operating_parameter(struct xcopy_fp_t *xfp, int is_target)
     }
     max_target_num = rcBuff[8] << 8 | rcBuff[9];
     max_segment_num = rcBuff[10] << 8 | rcBuff[11];
-    max_desc_len = rcBuff[12] << 24 | rcBuff[13] << 16 | rcBuff[14] << 8 | rcBuff[15];
+    max_desc_len = rcBuff[12] << 24 | rcBuff[13] << 16 | rcBuff[14] << 8 |
+                   rcBuff[15];
     max_segment_len = rcBuff[16] << 24 | rcBuff[17] << 16 |
         rcBuff[18] << 8 | rcBuff[19];
-    xfp->max_bytes = max_segment_len;
-    max_inline_data = rcBuff[20] << 24 | rcBuff[21] << 16 | rcBuff[22] << 8 | rcBuff[23];
+    xfp->max_bytes = max_segment_len ? max_segment_len : ULONG_MAX;
+    max_inline_data = rcBuff[20] << 24 | rcBuff[21] << 16 | rcBuff[22] << 8 |
+                      rcBuff[23];
     if (verbose) {
         printf(" >> Receive copy results (report operating parameters):\n");
         printf("    Maximum target descriptor count: %lu\n", max_target_num);
@@ -669,7 +683,8 @@ scsi_operating_parameter(struct xcopy_fp_t *xfp, int is_target)
     if (verbose) {
         printf("    Held data limit: %lu (usage: %d)\n",
                held_data_limit, list_id_usage);
-        num = rcBuff[28] << 24 | rcBuff[29] << 16 | rcBuff[30] << 8 | rcBuff[31];
+        num = rcBuff[28] << 24 | rcBuff[29] << 16 | rcBuff[30] << 8 |
+              rcBuff[31];
         printf("    Maximum stream device transfer size: %lu\n", num);
         printf("    Maximum concurrent copies: %u\n", rcBuff[36]);
         printf("    Data segment granularity: %u bytes\n", 1 << rcBuff[37]);
@@ -752,7 +767,8 @@ scsi_operating_parameter(struct xcopy_fp_t *xfp, int is_target)
             if (is_target && (xfp->sg_type & FT_ST))
                 valid++;
             if (verbose)
-                printf("        Copy block device with offset to stream device\n");
+                printf("        Copy block device with offset to stream "
+                       "device\n");
             break;
         case 0x09: /* copy stream device to block device with offset */
             if (!is_target && (xfp->sg_type & FT_ST))
@@ -760,15 +776,18 @@ scsi_operating_parameter(struct xcopy_fp_t *xfp, int is_target)
             if (is_target && (xfp->sg_type & FT_BLOCK))
                 valid++;
             if (verbose)
-                printf("        Copy stream device to block device with offset\n");
+                printf("        Copy stream device to block device with "
+                       "offset\n");
             break;
-        case 0x0a: /* copy block device with offset to block device with offset */
+        case 0x0a: /* copy block device with offset to block device with
+                    * offset */
             if (!is_target && (xfp->sg_type & FT_BLOCK))
                 valid++;
             if (is_target && (xfp->sg_type & FT_BLOCK))
                 valid++;
             if (verbose)
-                printf("        Copy block device with offset to block device with offset\n");
+                printf("        Copy block device with offset to block "
+                       "device with offset\n");
             break;
         case 0x0b: /* copy block device to stream device and hold data */
             if (!is_target && (xfp->sg_type & FT_BLOCK))
@@ -776,7 +795,8 @@ scsi_operating_parameter(struct xcopy_fp_t *xfp, int is_target)
             if (is_target && (xfp->sg_type & FT_ST))
                 valid++;
             if (verbose)
-                printf("        Copy block device to stream device and hold data\n");
+                printf("        Copy block device to stream device and hold "
+                       "data\n");
             break;
         case 0x0c: /* copy stream device to block device and hold data */
             if (!is_target && (xfp->sg_type & FT_ST))
@@ -784,7 +804,8 @@ scsi_operating_parameter(struct xcopy_fp_t *xfp, int is_target)
             if (is_target && (xfp->sg_type & FT_BLOCK))
                 valid++;
             if (verbose)
-                printf("        Copy stream device to block device and hold data\n");
+                printf("        Copy stream device to block device and hold "
+                       "data\n");
             break;
         case 0x0d: /* copy block device to block device and hold data */
             if (!is_target && (xfp->sg_type & FT_BLOCK))
@@ -792,7 +813,8 @@ scsi_operating_parameter(struct xcopy_fp_t *xfp, int is_target)
             if (is_target && (xfp->sg_type & FT_BLOCK))
                 valid++;
             if (verbose)
-                printf("        Copy block device to block device and hold data\n");
+                printf("        Copy block device to block device and hold "
+                       "data\n");
             break;
         case 0x0e: /* copy stream device to stream device and hold data */
             if (!is_target && (xfp->sg_type & FT_ST))
@@ -800,7 +822,8 @@ scsi_operating_parameter(struct xcopy_fp_t *xfp, int is_target)
             if (is_target && (xfp->sg_type & FT_ST))
                 valid++;
             if (verbose)
-                printf("        Copy block device to block device and hold data\n");
+                printf("        Copy block device to block device and hold "
+                       "data\n");
             break;
         case 0x0f: /* read from stream device and hold data */
             if (!is_target && (xfp->sg_type & FT_ST))
@@ -822,7 +845,8 @@ scsi_operating_parameter(struct xcopy_fp_t *xfp, int is_target)
             break;
         case 0xe2: /* FC N_Port_ID with N_Port_Name checking */
             if (verbose)
-                printf("        FC N_Port_ID with N_Port_Name target descriptor\n");
+                printf("        FC N_Port_ID with N_Port_Name target "
+                       "descriptor\n");
             td_list |= TD_FC_WWPN_AND_PORT;
             break;
         case 0xe3: /* Parallel Interface T_L  */
@@ -860,6 +884,21 @@ scsi_operating_parameter(struct xcopy_fp_t *xfp, int is_target)
                 printf("        SAS target descriptor\n");
             td_list |= TD_SAS;
             break;
+        case 0xea: /* IPv6 */
+            if (verbose)
+                printf("        IPv6 target descriptor\n");
+            td_list |= TD_IPV6;
+            break;
+        case 0xeb: /* IP Copy Service */
+            if (verbose)
+                printf("        IP Copy Service target descriptor\n");
+            td_list |= TD_IP_COPY_SERVICE;
+            break;
+        case 0xfe: /* ROD */
+            if (verbose)
+                printf("        ROD target descriptor\n");
+            td_list |= TD_ROD;
+            break;
         default:
             fprintf(stderr, ">> Unhandled target descriptor 0x%02x\n",
                    rcBuff[44 + n]);
@@ -1086,12 +1125,14 @@ desc_from_vpd_id(int sg_fd, unsigned char *desc, int desc_len,
         dStrHex((const char *)rcBuff, len, 1);
     }
 
-    while ((u = sg_vpd_dev_id_iter(rcBuff + 4, len - 4, &off, 0, -1, -1)) == 0) {
+    while ((u = sg_vpd_dev_id_iter(rcBuff + 4, len - 4, &off, 0, -1, -1)) ==
+           0) {
         ucp = rcBuff + 4 + off;
         i_len = ucp[3];
         if (((unsigned int)off + i_len + 4) > len) {
             fprintf(stderr, "    VPD page error: designator length %d longer "
-                    "than\n     remaining response length=%d\n", i_len, (len - off));
+                    "than\n     remaining response length=%d\n", i_len,
+                    (len - off));
             return SG_LIB_CAT_MALFORMED;
         }
         assoc = ((ucp[1] >> 4) & 0x3);
@@ -1336,14 +1377,17 @@ main(int argc, char * argv[])
     int blocks = 0;
     int num_xcopy = 0;
     int res, k;
-    int infd, outfd;
+    int infd, outfd, xcopy_fd;
     int ret = 0;
     unsigned char list_id = 1;
+    int list_id_given = 0;
     unsigned char src_desc[256];
     unsigned char dst_desc[256];
     int src_desc_len;
     int dst_desc_len;
     int seg_desc_type;
+    int on_src = 0;
+    int on_dst = 0;
 
     ifp.fname[0] = '\0';
     ofp.fname[0] = '\0';
@@ -1390,11 +1434,14 @@ main(int argc, char * argv[])
                 return SG_LIB_SYNTAX_ERROR;
             }
             list_id = (ret & 0xff);
+            list_id_given = 1;
         } else if (0 == strcmp(key, "id_usage")) {
             if (!strncmp(buf, "hold", 4))
                 list_id_usage = 0;
             else if (!strncmp(buf, "discard", 7))
                 list_id_usage = 2;
+            else if (!strncmp(buf, "disable", 7))
+                list_id_usage = 3;
             else {
                 fprintf(stderr, ME "bad argument to 'list_id_usage='\n");
                 return SG_LIB_SYNTAX_ERROR;
@@ -1474,6 +1521,10 @@ main(int argc, char * argv[])
             do_time = sg_get_num(buf);
         else if (0 == strncmp(key, "verb", 4))
             verbose = sg_get_num(buf);
+        else if (0 == strncmp(key, "--on_src", 8))
+            on_src = 1;
+        else if (0 == strncmp(key, "--on_dst", 8))
+            on_dst = 1;
         else if ((0 == strncmp(key, "--help", 7)) ||
                  (0 == strncmp(key, "-h", 2)) ||
                    (0 == strcmp(key, "-?"))) {
@@ -1483,12 +1534,32 @@ main(int argc, char * argv[])
                    (0 == strcmp(key, "-V"))) {
             fprintf(stderr, ME "%s\n", version_str);
             return 0;
-        } else {
+        } else if (0 == strncmp(key, "--verb", 6))
+            verbose += 1;
+        else if (0 == strcmp(key, "-vvvvv"))
+            verbose += 5;
+        else if (0 == strcmp(key, "-vvvv"))
+            verbose += 4;
+        else if (0 == strcmp(key, "-vvv"))
+            verbose += 3;
+        else if (0 == strcmp(key, "-vv"))
+            verbose += 2;
+        else if (0 == strcmp(key, "-v"))
+            verbose += 1;
+        else {
             fprintf(stderr, "Unrecognized option '%s'\n", key);
             fprintf(stderr, "For more information use '--help'\n");
             return SG_LIB_SYNTAX_ERROR;
         }
     }
+    if (on_src && on_dst) {
+        fprintf(stderr, "Syntax error - either specify --on_src OR "
+                "--on_dst\n");
+        fprintf(stderr, "For more information use '--help'\n");
+        return SG_LIB_SYNTAX_ERROR;
+    }
+    if (!on_src && !on_dst)
+        on_src = 1;
     if ((ibs && blk_sz && (ibs != blk_sz)) ||
         (obs && blk_sz && (obs != blk_sz))) {
         fprintf(stderr, "If 'ibs' or 'obs' given must be same as 'bs'\n");
@@ -1512,10 +1583,19 @@ main(int argc, char * argv[])
         fprintf(stderr, "bpt must be greater than 0\n");
         return SG_LIB_SYNTAX_ERROR;
     }
+    if (list_id_usage == 3) { /* list_id usage disabled */
+        if (!list_id_given)
+            list_id = 0;
+        if (list_id) {
+            fprintf(stderr, "list_id disabled by id_usage flag\n");
+            return SG_LIB_SYNTAX_ERROR;
+        }
+    }
 
 #ifdef SG_DEBUG
-    fprintf(stderr, ME "if=%s skip=%"PRId64" of=%s seek=%"PRId64" count=%"PRId64"\n",
-           ifp.fname, skip, ofp.fname, seek, dd_count);
+    fprintf(stderr, ME "%s if=%s skip=%" PRId64 " of=%s seek=%" PRId64
+            " count=%" PRId64 "\n", (on_src)?"on-source":"on-destination",
+            ifp.fname, skip, ofp.fname, seek, dd_count);
 #endif
     install_handler(SIGINT, interrupt_handler);
     install_handler(SIGQUIT, interrupt_handler);
@@ -1760,16 +1840,17 @@ main(int argc, char * argv[])
     fprintf(stderr,
             "Start of loop, count=%"PRId64", bpt=%d, "
             "lba_in=%"PRId64", lba_out=%"PRId64"\n",
-            dd_count, bpt, skip, skip + seek);
+            dd_count, bpt, skip, seek);
 #endif
+    xcopy_fd = (on_src) ? infd : outfd;
     while (dd_count > 0) {
         if (dd_count > bpt)
             blocks = bpt;
         else
             blocks = dd_count;
-        res = scsi_extended_copy(infd, list_id, src_desc, src_desc_len,
+        res = scsi_extended_copy(xcopy_fd, list_id, src_desc, src_desc_len,
                                  dst_desc, dst_desc_len, seg_desc_type,
-                                 blocks, skip, skip + seek);
+                                 blocks, skip, seek);
         if (res != 0)
             break;
         in_full += blocks;
